package com.huiju.estateagents.service.impl;

import cn.binarywang.wx.miniapp.api.WxMaService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.estateagents.channel.sysChannel.entity.TaChannelProxy;
import com.huiju.estateagents.channel.sysChannel.mapper.TaChannelProxyMapper;
import com.huiju.estateagents.common.StringUtils;
import com.huiju.estateagents.common.WxUtils;
import com.huiju.estateagents.entity.TaChannelAppRelation;
import com.huiju.estateagents.entity.TaMiniapp;
import com.huiju.estateagents.entity.TaMiniappTemplate;
import com.huiju.estateagents.mapper.TaChannelAppRelationMapper;
import com.huiju.estateagents.mapper.TaMiniappMapper;
import com.huiju.estateagents.mapper.TaMiniappTemplateMapper;
import com.huiju.estateagents.service.ITaMiniappService;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * <p>
 * 小程序表  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-09-11
 */
@Service
public class TaMiniappServiceImpl extends ServiceImpl<TaMiniappMapper, TaMiniapp> implements ITaMiniappService {
    @Autowired
    TaMiniappTemplateMapper taMiniappTemplateMapper;

    @Autowired
    WxUtils wxUtils;

    @Autowired
    private TaChannelAppRelationMapper taChannelAppRelationMapper;

    @Autowired
    private TaChannelProxyMapper taChannelProxyMapper;

    @Override
    public TaMiniapp saveInfoWithTpls(TaMiniapp taMiniapp, Integer channelId) throws Exception {
        if (StringUtils.isEmpty(taMiniapp.getMiniappId())) {
            throw new Exception("请填写小程序 AppID");
        }

        if (StringUtils.isEmpty(taMiniapp.getSecret())) {
            throw new Exception("请填写小程序 Secret");
        }

        if (StringUtils.isEmpty(taMiniapp.getName())) {
            throw new Exception("请填写小程序 名称");
        }

        if (null == taMiniapp.getOrgId() || taMiniapp.getOrgId() == 0) {
            throw new Exception("未知的小程序所属对象");
        }

        //判断渠道端新增小程序最大数
        QueryWrapper<TaChannelProxy> taChannelProxyQueryWrapper = new QueryWrapper<>();
        taChannelProxyQueryWrapper.eq("channel_id", channelId);
        TaChannelProxy taChannelProxy = taChannelProxyMapper.selectOne(taChannelProxyQueryWrapper);

        QueryWrapper<TaChannelAppRelation> relationQueryWrapper = new QueryWrapper<>();
        relationQueryWrapper.eq("channel_id", channelId);
        Integer miniAppCount = taChannelAppRelationMapper.selectCount(relationQueryWrapper);
        if (miniAppCount >= taChannelProxy.getAppMaxNum()){
            throw new Exception("当前系统仅供维护"+ taChannelProxy.getAppMaxNum() +"个小程序，需要维护更多项目，请联系相关管理人员");
        }

        //维护渠道和小程序关系
        TaChannelAppRelation taChannelAppRelation = new TaChannelAppRelation();
        taChannelAppRelation.setChannelId(channelId);
        taChannelAppRelation.setOrgId(taMiniapp.getOrgId());
        taChannelAppRelation.setCreateTime(LocalDateTime.now());
        taChannelAppRelationMapper.insert(taChannelAppRelation);


        TaMiniapp orign = this.getById(taMiniapp.getMiniappId());
        if (null != orign) {
            if (!this.updateById(taMiniapp)) {
                throw new Exception("保存小程序信息出错");
            }
        } else if (!this.save(taMiniapp)) {
            throw new Exception("保存小程序信息出错");
        }

        if (null != taMiniapp.getTpls()) {
            // 插入之前先删除
            QueryWrapper<TaMiniappTemplate> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("miniapp_id", taMiniapp.getMiniappId());
            taMiniappTemplateMapper.delete(queryWrapper);

            for(TaMiniappTemplate tpl: taMiniapp.getTpls()) {
                tpl.setMiniappId(taMiniapp.getMiniappId());
                taMiniappTemplateMapper.insert(tpl);
            }
        }

        WxMaService appService = wxUtils.getMiniApp(taMiniapp.getMiniappId());
        if (null == appService || (null != orign && !taMiniapp.getSecret().equals(orign.getSecret()))) {
            wxUtils.newService(taMiniapp, true);
        }

        return taMiniapp;
    }
}
